package postgres

import (
	"k8s.io/apimachinery/pkg/api/errors"
	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/fields"
	"k8s.io/apimachinery/pkg/watch"
	"k8s.io/client-go/kubernetes"
)

func (pg *Postgres) DestroyReplicas() error {
	pods, err := pg.GetPods(ReplicaRole)
	if err != nil {
		return err
	}
	client, err := kubernetes.NewForConfig(pg.restConfig)
	if err != nil {
		return err
	}
	for _, pod := range pods {
		for _, volume := range pod.Spec.Volumes {
			if volume.Name == "pgdata" {
				err := client.CoreV1().PersistentVolumeClaims(pg.GetNamespace()).Delete(pg.ctx, volume.PersistentVolumeClaim.ClaimName, v1.DeleteOptions{})
				if err != nil && !errors.IsNotFound(err) {
					return err
				}
			}
		}
		w, err := client.CoreV1().Pods(pg.GetNamespace()).Watch(pg.ctx, v1.ListOptions{
			FieldSelector: fields.OneTermEqualSelector("metadata.name", pod.Name).String(),
		})
		if err != nil {
			return err
		}
		defer w.Stop()

		err = client.CoreV1().Pods(pg.GetNamespace()).Delete(pg.ctx, pod.GetName(), v1.DeleteOptions{})
		if err != nil {
			return err
		}

		var deleted bool
		var added bool
		for event := range w.ResultChan() {
			if event.Type == watch.Deleted {
				deleted = true
			}
			if event.Type == watch.Added {
				added = true
			}
			if deleted && added {
				w.Stop()
			}
		}

		// A bug requires us to delete the pod 2 times to recreate the pvc.
		err = client.CoreV1().Pods(pg.GetNamespace()).Delete(pg.ctx, pod.GetName(), v1.DeleteOptions{})
		if err != nil {
			return err
		}

	}
	return nil
}
